# -*- coding: utf-8 -*-
import numpy as np
import re

def read_sms_mesh(mesh, nodestrings=False):

    """

    Reads in the SMS unstructured grid format. Also creates IDs for output to

    MIKE unstructured grid format.



    Parameters

    ----------

    mesh : str

        Full path to an SMS unstructured grid (.2dm) file.

    nodestrings : bool, optional

        Set to True to return the IDs of the node strings as a dictionary.



    Returns

    -------

    triangle : ndarray

        Integer array of shape (nele, 3). Each triangle is composed of

        three points and this contains the three node numbers (stored in

        nodes) which refer to the coordinates in X and Y (see below). Values

        are python-indexed.

    nodes : ndarray

        Integer number assigned to each node.

    X, Y, Z : ndarray

        Coordinates of each grid node and any associated Z value.

    types : ndarray

        Classification for each node string based on the number of node

        strings + 2. This is mainly for use if converting from SMS .2dm

        grid format to DHI MIKE21 .mesh format since the latter requires

        unique IDs for each boundary (with 0 and 1 reserved for land and

        sea nodes).

    nodestrings : list, optional (see nodestrings above)

        Optional list of lists containing the node IDs (python-indexed) of the

        node strings in the SMS grid.



    """



    fileRead = open(mesh, 'r')

    lines = fileRead.readlines()

    fileRead.close()



    triangles = []

    nodes = []

    types = []

    nodeStrings = []

    nstring = []

    x = []

    y = []

    z = []



    # MIKE unstructured grids allocate their boundaries with a type ID flag.

    # Although this function is not necessarily always the precursor to writing

    # a MIKE unstructured grid, we can create IDs based on the number of node

    # strings in the SMS grid. MIKE starts counting open boundaries from 2 (1

    # and 0 are land and sea nodes, respectively).

    typeCount = 2



    for line in lines:

        line = line.strip()

        if line.startswith('E3T'):

            ttt = line.split()

            t1 = int(ttt[2])

            t2 = int(ttt[3])

            t3 = int(ttt[4])

            triangles.append([t1, t2, t3])

        elif line.startswith('ND '):

            xy = line.split()

            x.append(float(xy[2]))

            y.append(float(xy[3]))

            z.append(float(xy[4]))

            nodes.append(int(xy[1]))

            # Although MIKE keeps zero and one reserved for normal nodes and

            # land nodes, SMS doesn't. This means it's not straightforward

            # to determine this information from the SMS file alone. It would

            # require finding nodes which are edge nodes and assigning their

            # ID to one. All other nodes would be zero until they were

            # overwritten when examining the node strings below.

            types.append(0)

        elif line.startswith('NS '):
            
            allTypes = re.split(' +',line)

            #allTypes = line.split(' ')



            for nodeID in enumerate(allTypes[1:]):
                #return nodes,types

                #types[np.abs(int(nodeID)) - 1] = typeCount
                #typeCount=0
                if nodeID[0]==0:
                    prev=int(allTypes[1])
                else:
                    prev=int(allTypes[1:][nodeID[0]-1])
               
                if int(nodeID[1]) > 0 and prev<0:
                    
                    nstring = []
                
                elif int(nodeID[1]) > 0:    
                    
                    nstring.append(int(nodeID[1])) 
                    
                else:

                    nstring.append(np.abs(int(nodeID[1])))

                    nodeStrings.append(nstring)



                # Count the number of node strings, and output that to types.

                # Nodes in the node strings are stored in nodeStrings.

#                if int(nodeID) < 0:
#
#                    typeCount += 1



    # Convert to numpy arrays.

    triangle = np.asarray(triangles)

    nodes = np.asarray(nodes)

    types = np.asarray(types)

    X = np.asarray(x)

    Y = np.asarray(y)

    Z = np.asarray(z)

    if nodestrings:

        return triangle, nodes, X, Y, Z, types, nodeStrings

    else:

        return triangle, nodes, X, Y, Z, types

      